@@ -1,13 +1,17 @@
 package CGI::Tiny;
 # ABSTRACT: Common Gateway Interface, with no frills
 
+# This file is part of CGI::Tiny which is released under:
+#   The Artistic License 2.0 (GPL Compatible)
+# See the documentation for CGI::Tiny for full license details.
+
 use strict;
 use warnings;
 use Carp ();
 use IO::Handle ();
 use Exporter ();
 
-our $VERSION = '0.018';
+our $VERSION = '0.019';
 
 use constant DEFAULT_REQUEST_BODY_LIMIT => 16777216;
 use constant DEFAULT_REQUEST_BODY_BUFFER => 262144;
@@ -175,27 +179,6 @@ sub server_port       { defined $ENV{SERVER_PORT} ? $ENV{SERVER_PORT} : '' }
 sub server_protocol   { defined $ENV{SERVER_PROTOCOL} ? $ENV{SERVER_PROTOCOL} : '' }
 sub server_software   { defined $ENV{SERVER_SOFTWARE} ? $ENV{SERVER_SOFTWARE} : '' }
 
-sub query_params      { [map { [@$_] } @{$_[0]->_query_params->{ordered}}] }
-sub query_param_names { [@{$_[0]->_query_params->{names}}] }
-sub query_param       { my $p = $_[0]->_query_params->{keyed}; exists $p->{$_[1]} ? $p->{$_[1]}[-1] : undef }
-sub query_param_array { my $p = $_[0]->_query_params->{keyed}; exists $p->{$_[1]} ? [@{$p->{$_[1]}}] : [] }
-
-sub _query_params {
-  my ($self) = @_;
-  unless (exists $self->{query_params}) {
-    $self->{query_params} = {names => \my @names, ordered => \my @ordered, keyed => \my %keyed};
-    foreach my $pair (split /[&;]/, $self->query) {
-      my ($name, $value) = split /=/, $pair, 2;
-      $value = '' unless defined $value;
-      do { tr/+/ /; s/%([0-9a-fA-F]{2})/chr hex $1/ge; utf8::decode $_ } for $name, $value;
-      push @names, $name unless exists $keyed{$name};
-      push @ordered, [$name, $value];
-      push @{$keyed{$name}}, $value;
-    }
-  }
-  return $self->{query_params};
-}
-
 sub headers {
   my ($self) = @_;
   unless (exists $self->{request_headers}) {
@@ -236,6 +219,38 @@ sub _cookies {
   return $self->{request_cookies};
 }
 
+sub params      { [map { [@$_] } @{$_[0]->_query_params->{ordered}}, @{$_[0]->_body_params->{ordered}}] }
+sub param_names { my $q = $_[0]->_query_params; [@{$q->{names}}, grep { !exists $q->{keyed}{$_} } @{$_[0]->_body_params->{names}}] }
+sub param       {
+  my ($self, $name) = @_;
+  my $p = $self->_body_params->{keyed};
+  return $p->{$name}[-1] if exists $p->{$name};
+  my $q = $self->_query_params->{keyed};
+  return exists $q->{$name} ? $q->{$name}[-1] : undef;
+}
+sub param_array { [map { exists $_->{$_[1]} ? @{$_->{$_[1]}} : () } $_[0]->_query_params->{keyed}, $_[0]->_body_params->{keyed}] }
+
+sub query_params      { [map { [@$_] } @{$_[0]->_query_params->{ordered}}] }
+sub query_param_names { [@{$_[0]->_query_params->{names}}] }
+sub query_param       { my $p = $_[0]->_query_params->{keyed}; exists $p->{$_[1]} ? $p->{$_[1]}[-1] : undef }
+sub query_param_array { my $p = $_[0]->_query_params->{keyed}; exists $p->{$_[1]} ? [@{$p->{$_[1]}}] : [] }
+
+sub _query_params {
+  my ($self) = @_;
+  unless (exists $self->{query_params}) {
+    $self->{query_params} = {names => \my @names, ordered => \my @ordered, keyed => \my %keyed};
+    foreach my $pair (split /[&;]/, $self->query) {
+      my ($name, $value) = split /=/, $pair, 2;
+      $value = '' unless defined $value;
+      do { tr/+/ /; s/%([0-9a-fA-F]{2})/chr hex $1/ge; utf8::decode $_ } for $name, $value;
+      push @names, $name unless exists $keyed{$name};
+      push @ordered, [$name, $value];
+      push @{$keyed{$name}}, $value;
+    }
+  }
+  return $self->{query_params};
+}
+
 sub body {
   my ($self) = @_;
   unless (exists $self->{body_content} or exists $self->{body_parts}) {
